@@ -1,6 +1,6 @@
 #!/bin/bash
 
-#    Copyright (C) 2008 Canonical Ltd.
+#    Copyright 2008-2011 Canonical Ltd.
 #
 #    This program is free software: you can redistribute it and/or modify
 #    it under the terms of the GNU General Public License version 3,
@@ -19,4 +19,14 @@ source "$TESTPATH/../testlib.sh"
 echo "Bug #237446" >> $TESTTMP/result
 do_cmd "0" --dry-run allow to 111.12.34.2/4
 
+# IPv6 Bugs
+sed -i 's/IPV6=no/IPV6=yes/' $TESTPATH/etc/default/ufw
+
+echo "proto ipv6 when IPV6=yes" >> $TESTTMP/result
+do_cmd "0" --dry-run allow to any proto ipv6
+
+sed -i 's/IPV6=yes/IPV6=no/' $TESTPATH/etc/default/ufw
+
+# End IPv6 Bugs
+
 exit 0
