@@ -689,6 +689,13 @@ class CommonTestCase(unittest.TestCase):
         self.assertEquals(self.rules['dapp'].dst, t[1])
         self.assertEquals("any", t[2])
         self.assertEquals("0.0.0.0/0", t[3])
+        r.set_direction("in")
+        t = r.get_app_tuple().split()
+        self.assertEquals(self.rules['dapp'].dapp, t[0])
+        self.assertEquals(self.rules['dapp'].dst, t[1])
+        self.assertEquals("any", t[2])
+        self.assertEquals("0.0.0.0/0", t[3])
+        self.assertEquals("in", t[4])
         r.set_interface("in", "eth0")
         t = r.get_app_tuple().split()
         self.assertEquals(self.rules['dapp'].dapp, t[0])
@@ -703,6 +710,13 @@ class CommonTestCase(unittest.TestCase):
         self.assertEquals("0.0.0.0/0", t[1])
         self.assertEquals(self.rules['sapp'].sapp, t[2])
         self.assertEquals(self.rules['sapp'].src, t[3])
+        r.set_direction("out")
+        t = r.get_app_tuple().split()
+        self.assertEquals("any", t[0])
+        self.assertEquals("0.0.0.0/0", t[1])
+        self.assertEquals(self.rules['sapp'].sapp, t[2])
+        self.assertEquals(self.rules['sapp'].src, t[3])
+        self.assertEquals("out", t[4])
         r.set_interface("out", "eth0")
         t = r.get_app_tuple().split()
         self.assertEquals("any", t[0])
